# -*- coding: utf-8 -*-
from test_bitbucketbase import BitbucketFixture
import json

import httpretty
from uritemplate import expand
from pybitbucket.repository import (
    RepositoryRole, RepositoryType, RepositoryForkPolicy,
    Repository, RepositoryV1,
    RepositoryPayload)
from pybitbucket.team import Team
from pybitbucket.bitbucket import Bitbucket
from pybitbucket.user import User
from pybitbucket.commit import Commit
from pybitbucket.pullrequest import PullRequest
from voluptuous import MultipleInvalid


class RepositoryFixture(BitbucketFixture):
    # GIVEN: a class under test
    class_under_test = 'Repository'

    # GIVEN: An example object created from example data
    @classmethod
    def example_object(cls):
        return Repository(
            json.loads(cls.resource_data()),
            client=cls.test_client)

    # GIVEN: Example data attributes for a repository
    owner = 'teamsinspace'
    name = 'teamsinspace.bitbucket.org'
    description = 'Description'
    full_name = owner + '/' + name
    language = 'python'
    scm = RepositoryType.GIT
    fork_policy = RepositoryForkPolicy.ALLOW_FORKS
    role = RepositoryRole.OWNER
    is_private = False
    has_issues = True
    has_wiki = True


class RepositoryPayloadFixture(RepositoryFixture):
    builder = RepositoryPayload()

    # GIVEN: a class under test
    class_under_test = 'RepositoryPayload'

    # GIVEN: An example object created from example data
    @classmethod
    def example_object(cls):
        return RepositoryPayload(json.loads(cls.resource_data()))


class RepositoryV1Fixture(RepositoryFixture):
    # GIVEN: a class under test
    class_under_test = 'RepositoryV1'

    # GIVEN: An example object created from example data
    @classmethod
    def example_object(cls):
        return RepositoryV1(
            json.loads(cls.resource_data()),
            client=cls.test_client)

    # GIVEN: Example data attributes for a repository
    full_name = 'teamsinspace/teamsinspace.bitbucket.org'


class TestGettingTheStringRepresentation(RepositoryFixture):
    @classmethod
    def setup_class(cls):
        cls.repository_str = str(cls.example_object())

    def test_string_is_not_the_default_format(self):
        assert not self.repository_str.startswith('<')
        assert not self.repository_str.endswith('>')

    def test_string_has_the_class_name_and_id_attribute(self):
        assert self.repository_str.startswith('Repository full_name:')


class TestCheckingTheExampleData(RepositoryFixture):
    @classmethod
    def setup_class(cls):
        cls.data = json.loads(cls.resource_data())

    def test_passes_the_type_check(self):
        assert Repository.is_type(self.data)


class TestCheckingTheExampleDataForV1(RepositoryV1Fixture):
    @classmethod
    def setup_class(cls):
        cls.data = json.loads(cls.resource_data())

    def test_passes_the_type_check(self):
        assert RepositoryV1.is_type(self.data)


class TestAccessingRepositoryAttributes(RepositoryFixture):
    @classmethod
    def setup_class(cls):
        cls.response = cls.example_object()

    def test_common_attributes_are_valid(self):
        assert self.name == self.response.name
        assert self.scm == self.response.scm
        assert self.language == self.response.language

    def test_owner_is_a_team(self):
        assert isinstance(self.response.owner, Team)

    def test_clone_is_a_git_url(self):
        assert (
            'https://ianbuchanan@bitbucket.org/' +
            'teamsinspace/teamsinspace.bitbucket.org.git') == \
                self.response.clone['https']


class TestAccessingRepositoryV1Attributes(RepositoryV1Fixture):
    @classmethod
    def setup_class(cls):
        cls.response = cls.example_object()

    def test_common_attributes_are_valid(self):
        assert self.name == self.response.name
        assert self.scm == self.response.scm
        assert self.language == self.response.language


class TestDeleting(RepositoryFixture):
    @httpretty.activate
    def test_response_is_not_an_exception(self):
        httpretty.register_uri(
            httpretty.DELETE,
            self.resource_url(),
            status=204)
        result = self.example_object().delete()
        assert result is None


class TestCreatingNewRepository(RepositoryFixture):
    @classmethod
    def setup_class(cls):
        cls.url = expand(
            Repository.templates['create'], {
                'bitbucket_url': cls.test_client.get_bitbucket_url(),
                'owner': cls.test_client.get_username(),
                'repository_name': cls.name,
            })

    @httpretty.activate
    def test_response_is_a_repository(self):
        httpretty.register_uri(
            httpretty.POST,
            self.url,
            content_type='application/json',
            body=self.resource_data(),
            status=200)
        payload = RepositoryPayload() \
            .add_name(self.name) \
            .add_fork_policy(self.fork_policy) \
            .add_is_private(self.is_private)
        response = Repository.create(payload, client=self.test_client)
        assert 'application/json' == \
            httpretty.last_request().headers.get('Content-Type')
        assert isinstance(response, Repository)


class TestForkingRepository(RepositoryFixture):
    @classmethod
    def setup_class(cls):
        cls.url = expand(
            Repository.templates['fork'], {
                'bitbucket_url': cls.test_client.get_bitbucket_url(),
                'owner': cls.test_client.get_username(),
                'repository_name': cls.name,
            })

    @httpretty.activate
    def test_response_is_a_repository(self):
        httpretty.register_uri(
            httpretty.POST,
            self.url,
            content_type='application/json',
            body=self.resource_data(),
            status=200)
        payload = RepositoryPayload() \
            .add_name(self.name) \
            .add_fork_policy(self.fork_policy) \
            .add_is_private(self.is_private)
        response = Repository.fork(payload, client=self.test_client)
        assert 'application/x-www-form-urlencoded' == \
            httpretty.last_request().headers.get('Content-Type')
        assert isinstance(response, Repository)


class TestFindingRepositoryByFullName(RepositoryFixture):
    @httpretty.activate
    def test_response_is_a_repository(self):
        httpretty.register_uri(
            httpretty.GET,
            self.resource_url(),
            content_type='application/json',
            body=self.resource_data(),
            status=200)
        response = Repository.find_repository_by_full_name(
            full_name=self.full_name,
            client=self.test_client)
        assert isinstance(response, Repository)


class TestFindingRepositoryByNameOnly(RepositoryFixture):
    @classmethod
    def setup_class(cls):
        template = (
            Bitbucket(client=cls.test_client)
            .data
            .get('_links', {})
            .get('repositoryByOwnerAndRepositoryName', {})
            .get('href'))
        cls.url = expand(
            template, {
                'bitbucket_url': cls.test_client.get_bitbucket_url(),
                'owner': cls.test_client.get_username(),
                'repository_name': cls.name
            })

    @httpretty.activate
    def test_response_is_a_repository(self):
        httpretty.register_uri(
            httpretty.GET,
            self.url,
            content_type='application/json',
            body=self.resource_data(),
            status=200)
        response = Repository.find_repository_by_name_and_owner(
            repository_name=self.name,
            client=self.test_client)
        assert isinstance(response, Repository)


class TestFindingRepositoriesForOwner(RepositoryFixture):
    @classmethod
    def setup_class(cls):
        template = (
            Bitbucket(client=cls.test_client)
            .data
            .get('_links', {})
            .get('repositoriesByOwnerAndRole', {})
            .get('href'))
        cls.url = expand(
            template, {
                'bitbucket_url': cls.test_client.get_bitbucket_url(),
                'owner': cls.owner
            })

    @httpretty.activate
    def test_response_from_only_owner_is_a_repository_generator(self):
        httpretty.register_uri(
            httpretty.GET,
            self.url,
            content_type='application/json',
            body=self.resource_list_data(),
            status=200)
        response = Repository.find_repositories_by_owner_and_role(
            owner=self.owner,
            client=self.test_client)
        assert isinstance(next(response), Repository)

    @httpretty.activate
    def test_response_from_owner_and_role_is_a_repository_generator(self):
        httpretty.register_uri(
            httpretty.GET,
            self.url,
            content_type='application/json',
            body=self.resource_list_data(),
            status=200)
        response = Repository.find_repositories_by_owner_and_role(
            owner=self.owner,
            role=RepositoryRole.MEMBER,
            client=self.test_client)
        assert isinstance(next(response), Repository)


class TestFindingRepositoriesForSelf(RepositoryFixture):
    @classmethod
    def setup_class(cls):
        template = (
            Bitbucket(client=cls.test_client)
            .data
            .get('_links', {})
            .get('repositoriesByOwnerAndRole', {})
            .get('href'))
        cls.url = expand(
            template, {
                'bitbucket_url': cls.test_client.get_bitbucket_url(),
                'owner': cls.test_client.get_username()
            })

    @httpretty.activate
    def test_response_from_no_args_is_a_repository_generator(self):
        httpretty.register_uri(
            httpretty.GET,
            self.url,
            content_type='application/json',
            body=self.resource_list_data(),
            status=200)
        response = Repository.find_repositories_by_owner_and_role(
            client=self.test_client)
        assert isinstance(next(response), Repository)

    @httpretty.activate
    def test_response_from_role_only_is_a_repository_generator(self):
        httpretty.register_uri(
            httpretty.GET,
            self.url,
            content_type='application/json',
            body=self.resource_list_data(),
            status=200)
        response = Repository.find_repositories_by_owner_and_role(
            role=RepositoryRole.MEMBER,
            client=self.test_client)
        assert isinstance(next(response), Repository)


class TestFindingPublicRepositories(RepositoryFixture):
    @classmethod
    def setup_class(cls):
        template = (
            Bitbucket(client=cls.test_client)
            .data
            .get('_links', {})
            .get('repositoriesThatArePublic', {})
            .get('href'))
        cls.url = expand(
            template, {
                'bitbucket_url': cls.test_client.get_bitbucket_url()
            })

    @httpretty.activate
    def test_response_is_a_repository_generator(self):
        httpretty.register_uri(
            httpretty.GET,
            self.url,
            content_type='application/json',
            body=self.resource_list_data(),
            status=200)
        response = Repository.find_public_repositories(
            client=self.test_client)
        assert isinstance(next(response), Repository)


class TestAccessingLinks(RepositoryFixture):
    @classmethod
    def get_link_url(cls, name):
        return (
            cls.response
            .data
            .get('links', {})
            .get(name, {})
            .get('href'))

    @classmethod
    def setup_class(cls):
        cls.response = cls.example_object()
        cls.forks_url = cls.get_link_url('forks')
        cls.forks_data = cls.resource_list_data()
        cls.watchers_url = cls.get_link_url('watchers')
        cls.watchers_data = cls.resource_list_data('User')
        cls.commits_url = cls.get_link_url('commits')
        cls.commits_data = cls.resource_list_data('Commit')
        cls.pullrequests_url = cls.get_link_url('pullrequests')
        cls.pullrequests_data = cls.resource_list_data('PullRequest')

    @httpretty.activate
    def test_forks_returns_a_repository_generator(self):
        httpretty.register_uri(
            httpretty.GET,
            self.forks_url,
            content_type='application/json',
            body=self.forks_data,
            status=200)
        response = self.response.forks()
        assert isinstance(next(response), Repository)

    @httpretty.activate
    def test_watchers_returns_a_user_generator(self):
        httpretty.register_uri(
            httpretty.GET,
            self.watchers_url,
            content_type='application/json',
            body=self.watchers_data,
            status=200)
        response = self.response.watchers()
        assert isinstance(next(response), User)

    @httpretty.activate
    def test_commits_returns_a_commit_generator(self):
        httpretty.register_uri(
            httpretty.GET,
            self.commits_url,
            content_type='application/json',
            body=self.commits_data,
            status=200)
        response = self.response.commits()
        assert isinstance(next(response), Commit)

    @httpretty.activate
    def test_pullrequests_returns_a_pullrequest_generator(self):
        httpretty.register_uri(
            httpretty.GET,
            self.pullrequests_url,
            content_type='application/json',
            body=self.pullrequests_data,
            status=200)
        response = self.response.pullrequests()
        assert isinstance(next(response), PullRequest)


class TestNavigatingToIssues(RepositoryFixture):
    @classmethod
    def setup_class(cls):
        cls.response = cls.example_object()
        cls.issues_url = cls.response.v1.links.get('issues', {}).get('href')
        cls.issues_data = cls.resource_list_data('Issue')

    @httpretty.activate
    def test_attributes_are_not_empty(self):
        httpretty.register_uri(
            httpretty.GET,
            self.issues_url,
            content_type='application/json',
            body=self.issues_data,
            status=200)
        assert list(self.response.v1.issues())
        # There's no type for Issues yet.
        # assert isinstance(next(self.response.v1.issues()), Issue)


class TestNavigatingFromV1toV2(RepositoryV1Fixture):
    @classmethod
    def setup_class(cls):
        cls.response = cls.example_object()
        repository_template = (
            Bitbucket(client=cls.test_client)
            .data
            .get('_links', {})
            .get('repositoryByOwnerAndRepositoryName', {})
            .get('href'))
        cls.repository_url = expand(
            repository_template, {
                'bitbucket_url': cls.test_client.get_bitbucket_url(),
                'owner': cls.owner,
                'repository_name': cls.name
            })
        cls.repository_data = cls.resource_list_data('Repository')
        user_template = (
            Bitbucket(client=cls.test_client)
            .data
            .get('_links', {})
            .get('userByUsername', {})
            .get('href'))
        cls.user_url = expand(
            user_template, {
                'bitbucket_url': cls.test_client.get_bitbucket_url(),
                'username': cls.owner,
            })
        cls.user_data = cls.resource_list_data('User')

    @httpretty.activate
    def test_v2_self_returns_a_repository(self):
        print('expected: {}'.format(self.repository_url))
        httpretty.register_uri(
            httpretty.GET,
            self.repository_url,
            content_type='application/json',
            body=self.repository_data,
            status=200)
        response = self.response.v2.self()
        assert isinstance(response, Repository)

    @httpretty.activate
    def test_v2_owner_returns_a_user(self):
        httpretty.register_uri(
            httpretty.GET,
            self.user_url,
            content_type='application/json',
            body=self.user_data,
            status=200)
        response = self.response.v2.owner()
        assert isinstance(response, User)


class TestCreatingDefaultRepositoryPayload(RepositoryPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.data = cls.builder.build()

    def test_default_payload_is_empty_dictionary(self):
        assert {} == self.data

    def test_default_is_invalid(self):
        try:
            self.builder.validate()
            assert False
        except Exception as e:
            assert isinstance(e, MultipleInvalid)


class TestAddingIsPrivateToRepositoryPayload(RepositoryPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_is_private = cls.builder.add_is_private(cls.is_private)

    def test_immutability_on_adding_is_private(self):
        assert self.with_is_private
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'is_private': self.is_private
        }
        assert expected == self.with_is_private.build()


class TestAddingForkPolicyToRepositoryPayload(RepositoryPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_fork_policy = cls.builder.add_fork_policy(cls.fork_policy)

    def test_immutability_on_adding_a_fork_policy(self):
        assert self.with_fork_policy
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'fork_policy': self.fork_policy
        }
        assert expected == self.with_fork_policy.build()


class TestAddingScmToRepositoryPayload(RepositoryPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_scm = cls.builder.add_scm(cls.scm)

    def test_immutability_on_adding_scm(self):
        assert self.with_scm
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'scm': self.scm
        }
        assert expected == self.with_scm.build()


class TestAddingNameToRepositoryPayload(RepositoryPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_name = cls.builder.add_name(cls.name)

    def test_immutability_on_adding_a_name(self):
        assert self.with_name
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'name': self.name
        }
        assert expected == self.with_name.build()


class TestAddingDescriptionToRepositoryPayload(RepositoryPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_description = cls.builder.add_description(cls.description)

    def test_immutability_on_adding_a_description(self):
        assert self.with_description
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'description': self.description
        }
        assert expected == self.with_description.build()


class TestAddingLanguageToRepositoryPayload(RepositoryPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_language = cls.builder.add_language(cls.language)

    def test_immutability_on_adding_a_language(self):
        assert self.with_language
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'language': self.language
        }
        assert expected == self.with_language.build()


class TestAddingHasIssuesToRepositoryPayload(RepositoryPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_has_issues = cls.builder.add_has_issues(cls.has_issues)

    def test_immutability_on_adding_a_has_issues(self):
        assert self.with_has_issues
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'has_issues': self.has_issues
        }
        assert expected == self.with_has_issues.build()


class TestAddingHasWikiToRepositoryPayload(RepositoryPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_has_wiki = cls.builder.add_has_wiki(cls.has_wiki)

    def test_immutability_on_adding_a_has_wiki(self):
        assert self.with_has_wiki
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'has_wiki': self.has_wiki
        }
        assert expected == self.with_has_wiki.build()


class TestCreatingMinimalRepositoryPayload(RepositoryPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.payload = RepositoryPayload() \
            .add_fork_policy(cls.fork_policy) \
            .add_is_private(cls.is_private)
        cls.expected = json.loads(cls.resource_data(
            'RepositoryPayload.minimal'))

    def test_minimum_viable_payload_structure_for_create(self):
        assert self.payload.validate().build() == self.expected


class TestCreatingFullRepositoryPayload(RepositoryPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.payload = RepositoryPayload() \
            .add_owner(cls.owner) \
            .add_name(cls.name) \
            .add_description(cls.description) \
            .add_scm(RepositoryType.GIT) \
            .add_fork_policy(cls.fork_policy) \
            .add_is_private(cls.is_private) \
            .add_has_issues(cls.has_issues) \
            .add_has_wiki(cls.has_wiki) \
            .add_language(cls.language)
        cls.expected = json.loads(cls.resource_data(
            'RepositoryPayload.full'))

    def test_full_payload_structure(self):
        assert self.payload.validate().build() == self.expected



# -*- coding: utf-8 -*-
import httpretty

from pybitbucket.bitbucket import Client, BadRequestError, ServerError

from test_auth import FakeAuth


class TestClient(object):

    @httpretty.activate
    def test_exceptions(self):
        a = FakeAuth()
        session = a.start_http_session()

        httpretty.register_uri(
            httpretty.GET,
            a.server_base_uri,
            status=400)
        response = session.get(a.server_base_uri)
        try:
            raise BadRequestError(response)
        except BadRequestError as e:
            assert e
        else:
            assert False

        httpretty.register_uri(
            httpretty.GET,
            a.server_base_uri,
            status=500)
        response = session.get(a.server_base_uri)
        try:
            raise ServerError(response)
        except ServerError as e:
            assert e
        else:
            assert False

    @httpretty.activate
    def test_expect_ok(self):
        a = FakeAuth()
        httpretty.register_uri(httpretty.GET, a.server_base_uri)
        session = a.start_http_session()
        response = session.get(a.server_base_uri)
        try:
            Client.expect_ok(response)
        except Exception:
            assert False

    @httpretty.activate
    def test_structured_exception(self):
        a = FakeAuth()
        client = Client(a)
        http_error_code = 400
        url = a.server_base_uri + '/1.0/user'
        example = '''{"error": {"message": "Repository already exists."}}'''
        httpretty.register_uri(
            httpretty.GET,
            url,
            content_type='application/json',
            body=example,
            status=http_error_code)
        response = client.session.get(url)
        try:
            Client.expect_ok(response)
        except BadRequestError as b:
            assert b.url == url
            assert b.code == http_error_code
            assert b.error_message == "Repository already exists."
            assert list(b.error)
        else:
            assert False

    @httpretty.activate
    def test_client_construction(self):
        client = Client(FakeAuth())
        assert 'https://staging.bitbucket.org/api' == \
            client.get_bitbucket_url()
        url = client.get_bitbucket_url() + '/1.0/user'
        httpretty.register_uri(httpretty.GET, url)
        response = client.session.get(url)
        assert 200 == response.status_code



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""
Provides classes for manipulating Snippet resources.

Classes:
- SnippetRole: enumerates the possible roles a user can have with a snippet
- SnippetPayload: encapsulates payload for creating and modifying snippets
- Snippet: represents a snippet
"""

from uritemplate import expand
from voluptuous import Schema, Optional, In

from pybitbucket.bitbucket import (
    Bitbucket, BitbucketBase, Client, PayloadBuilder, RepositoryType, Enum)


def open_files(filelist):
    files = []
    for filename in filelist:
        files.append(('file', (filename, open(filename, 'rb'))))
    return files


class SnippetRole(Enum):
    OWNER = 'owner'
    CONTRIBUTOR = 'contributor'
    MEMBER = 'member'


class SnippetPayload(PayloadBuilder):
    """
    A builder object to help create payloads
    for creating and updating snippets.
    """

    schema = Schema({
        Optional('title'): str,
        Optional('scm'): In(RepositoryType),
        Optional('is_private'): bool
    })

    def __init__(
            self,
            payload=None,
            owner=None):
        super(SnippetPayload, self).__init__(payload=payload)
        self._owner = owner

    @property
    def owner(self):
        return self._owner

    def add_owner(self, owner):
        return SnippetPayload(
            payload=self._payload.copy(),
            owner=owner)

    def add_title(self, title):
        new = self._payload.copy()
        new['title'] = title
        return SnippetPayload(
            payload=new,
            owner=self.owner)

    def add_scm(self, scm):
        new = self._payload.copy()
        new['scm'] = scm
        return SnippetPayload(
            payload=new,
            owner=self.owner)

    def add_is_private(self, is_private):
        new = self._payload.copy()
        new['is_private'] = is_private
        return SnippetPayload(
            payload=new,
            owner=self.owner)


class Snippet(BitbucketBase):
    """Represents a snippet."""

    id_attribute = 'id'
    resource_type = 'snippets'
    templates = {
        'create': '{+bitbucket_url}/2.0/snippets'
    }

    @staticmethod
    def is_type(data):
        # Snippet URLs look like this:
        # https://api.bitbucket.org/2.0/snippets/pybitbucket/Xqoz8
        # Which doesn't follow the pattern of:
        # resource_type/id_attribute
        # So we can't use `has_v2_self_url` to categorize.
        if (
                (data.get('links') is None) or
                (data['links'].get('self') is None) or
                (data['links']['self'].get('href') is None) or
                (data.get(Snippet.id_attribute) is None)):
            return False
        # Since the structure is right, assume it is v2.
        is_v2 = True
        url_path = data['links']['self']['href'].split('/')
        # Start looking from the end of the path.
        position = -1
        is_v2 = is_v2 and (data[Snippet.id_attribute] == url_path[position])
        # After matching the id_attribute,
        # skip a position for the account name.
        # The resource_type should be the preceding part of the path.
        position -= 2
        is_v2 = (Snippet.resource_type == url_path[position])
        return is_v2

    def __init__(self, data, client=Client()):
        super(Snippet, self).__init__(data, client=client)
        if data.get('files'):
            self.filenames = [str(f) for f in data['files']]
        # TODO: Snippet has patch & diff links but I don't know what they do.

    @classmethod
    def create(cls, files, payload=None, client=None):
        """Create a new snippet.

        :param files:
        :type files:
        :param payload: the options for creating the new snippet.
        :type payload: SnippetPayload
        :param client: the configured connection to Bitbucket.
            If not provided, assumes an Anonymous connection.
        :type client: bitbucket.Client
        :returns: the new build status object.
        :rtype: BuildStatus
        :raises: ValueError
        """
        client = client or Client()
        payload = payload or SnippetPayload()
        json = payload.validate().build()
        api_url = expand(
            cls.templates['create'], {
                'bitbucket_url': client.get_bitbucket_url(),
            })
        return cls.post(api_url, json=json, files=files, client=client)

    def modify(self, files=None, payload=None):
        """
        A convenience method for changing the current snippet.
        The parameters make it easier to know what can be changed
        and allow references with file names instead of File objects.
        """
        files = files or open_files([])
        payload = payload or SnippetPayload()
        json = payload.validate().build()
        return self.put(json=json, files=files)

    def content(self, filename):
        """
        A method for obtaining the contents of a file on a snippet.
        If the filename is not on the snippet, no content is returned.
        """
        if not self.files.get(filename):
            return
        url = self.files[filename]['links']['self']['href']
        response = self.client.session.get(url)
        Client.expect_ok(response)
        return response.content

    @staticmethod
    def find_snippets_for_role(role=SnippetRole.OWNER, client=None):
        """
        A convenience method for finding snippets by the user's role.
        The method is a generator Snippet objects.

        :param role: the role of the current user on the snippets.
            If not provided, assumes the relationship owner.
        :type role: SnippetRole
        :param client: the configured connection to Bitbucket.
            If not provided, assumes an Anonymous connection.
        :type client: bitbucket.Client
        :returns: an iterator over the selected snippets.
        :rtype: iterator
        """
        client = client or Client()
        SnippetRole(role)
        return Bitbucket(client=client).snippetsForRole(role=role)

    @staticmethod
    def find_snippet_by_id_and_owner(id, owner=None, client=None):
        """
        A convenience method for finding a specific snippet.
        In contrast to the pure hypermedia driven method on the Bitbucket
        class, this method returns a Snippet object, instead of the
        generator.

        :param id: the id of the snippet.
        :type id: str
        :param owner: the owner of the snippet.
            If not provided, assumes the current user.
        :type owner: str
        :param client: the configured connection to Bitbucket.
            If not provided, assumes an Anonymous connection.
        :type client: bitbucket.Client
        :returns: the snippet referenced by the id.
        :rtype: bitbucket.Snippet
        """
        client = client or Client()
        owner = owner or client.get_username()
        return next(Bitbucket(client=client).snippetByOwnerAndSnippetId(
            owner=owner,
            snippet_id=id))


Client.bitbucket_types.add(Snippet)



# -*- coding: utf-8 -*-
from test_bitbucketbase import BitbucketFixture
import json

import httpretty
from uritemplate import expand
from pybitbucket.build import (
    BuildStatus, BuildStatusStates, BuildStatusPayload)
from voluptuous import MultipleInvalid


class BuildStatusFixture(BitbucketFixture):
    # GIVEN: a class under test
    class_under_test = 'BuildStatus'

    # GIVEN: An example object created from example data
    @classmethod
    def example_object(cls):
        return BuildStatus(
            json.loads(cls.resource_data()),
            client=cls.test_client)

    # GIVEN: Example data attributes for a build status
    owner = 'emmap1'
    repository_name = 'MyRepo'
    revision = '61d9e64348f9da407e62f64726337fd3bb24b466'
    key = 'BAMBOO-PROJECT-X'
    state = BuildStatusStates.SUCCESSFUL
    url = 'https://example.com/path/to/build'
    name = 'Build #34'
    description = 'Changes by John Doe'


class BuildStatusPayloadFixture(BuildStatusFixture):
    builder = BuildStatusPayload()

    # GIVEN: a class under test
    class_under_test = 'BuildStatusPayload'

    # GIVEN: An example object created from example data
    @classmethod
    def example_object(cls):
        return BuildStatusPayload(json.loads(cls.resource_data()))


class TestGettingTheStringRepresentation(BuildStatusFixture):
    @classmethod
    def setup_class(cls):
        cls.buildstatus_str = str(cls.example_object())

    def test_string_is_not_the_default_format(self):
        assert not self.buildstatus_str.startswith('<')
        assert not self.buildstatus_str.endswith('>')

    def test_string_has_the_class_name_and_id_attribute(self):
        assert self.buildstatus_str.startswith('BuildStatus key:')


class TestCheckingTheExampleData(BuildStatusFixture):
    @classmethod
    def setup_class(cls):
        cls.data = json.loads(cls.resource_data())

    def test_passes_the_type_check(self):
        assert BuildStatus.is_type(self.data)


class TestAccessingBuildStatusAttributes(BuildStatusFixture):
    @classmethod
    def setup_class(cls):
        cls.response = cls.example_object()

    def test_common_attributes_are_valid(self):
        assert self.state == self.response.state
        assert self.key == self.response.key
        assert self.name == self.response.name
        assert self.url == self.response.url
        assert self.description == self.response.description


class TestDeleting(BuildStatusFixture):
    @httpretty.activate
    def test_response_is_not_an_exception(self):
        httpretty.register_uri(
            httpretty.DELETE,
            self.resource_url(),
            status=204)
        result = self.example_object().delete()
        assert result is None


class TestCreatingNewBuildStatus(BuildStatusFixture):
    @classmethod
    def setup_class(cls):
        cls.url = expand(
            BuildStatus.templates['create'], {
                'bitbucket_url': cls.test_client.get_bitbucket_url(),
                'owner': cls.test_client.get_username(),
                'repository_name': cls.repository_name,
                'revision': cls.revision
            })

    @httpretty.activate
    def test_response_is_a_buildstatus(self):
        httpretty.register_uri(
            httpretty.POST,
            self.url,
            content_type='application/json',
            body=self.resource_data(),
            status=200)
        payload = BuildStatusPayload() \
            .add_key(self.key) \
            .add_state(self.state) \
            .add_url(self.url) \
            .add_name(self.name) \
            .add_description(self.description) \
            .add_repository_name(self.repository_name) \
            .add_revision(self.revision)
        response = BuildStatus.create(payload, client=self.test_client)
        assert 'application/json' == \
            httpretty.last_request().headers.get('Content-Type')
        assert isinstance(response, BuildStatus)


class TestModifyingBuildStatus(BuildStatusFixture):
    @classmethod
    def setup_class(cls):
        cls.url = cls.example_object().links['self']['href']

    @httpretty.activate
    def test_response_is_a_buildstatus(self):
        httpretty.register_uri(
            httpretty.PUT,
            self.url,
            content_type='application/json',
            body=self.resource_data(),
            status=200)
        payload = BuildStatusPayload() \
            .add_key(self.key) \
            .add_state(self.state) \
            .add_url(self.url) \
            .add_name(self.name) \
            .add_description(self.description)
        response = self.example_object().modify(payload)
        assert 'application/json' == \
            httpretty.last_request().headers.get('Content-Type')
        assert isinstance(response, BuildStatus)


class TestCreatingDefaultBuildStatusPayload(BuildStatusPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.data = cls.builder.build()

    def test_default_payload_is_empty_dictionary(self):
        assert {} == self.data

    def test_default_is_invalid(self):
        try:
            self.builder.validate()
            assert False
        except Exception as e:
            assert isinstance(e, MultipleInvalid)


class TestAddingKeyToBuildStatusPayload(BuildStatusPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_key = cls.builder.add_key(cls.key)

    def test_immutability_after_add(self):
        assert self.with_key
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'key': self.key
        }
        assert expected == self.with_key.build()


class TestAddingStateToBuildStatusPayload(BuildStatusPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_state = cls.builder.add_state(cls.state)

    def test_immutability_after_add(self):
        assert self.with_state
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'state': self.state
        }
        assert expected == self.with_state.build()


class TestAddingUrlToBuildStatusPayload(BuildStatusPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_url = cls.builder.add_url(cls.url)

    def test_immutability_after_add(self):
        assert self.with_url
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'url': self.url
        }
        assert expected == self.with_url.build()


class TestAddingNameToBuildStatusPayload(BuildStatusPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_name = cls.builder.add_name(cls.name)

    def test_immutability_after_add(self):
        assert self.with_name
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'name': self.name
        }
        assert expected == self.with_name.build()


class TestAddingDescriptionToBuildStatusPayload(BuildStatusPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_description = cls.builder.add_description(cls.description)

    def test_immutability_after_add(self):
        assert self.with_description
        assert {} == self.builder.build()

    def test_structure(self):
        expected = {
            'description': self.description
        }
        assert expected == self.with_description.build()


class TestAddingOwnerToBuildStatusPayload(BuildStatusPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_owner = cls.builder.add_owner(cls.owner)

    def test_immutability_after_add(self):
        assert self.with_owner
        assert {} == self.builder.build()

    def test_structure(self):
        assert self.owner == self.with_owner.owner


class TestAddingRevisionToBuildStatusPayload(BuildStatusPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_revision = cls.builder.add_revision(cls.revision)

    def test_immutability_after_add(self):
        assert self.with_revision
        assert {} == self.builder.build()

    def test_structure(self):
        assert self.revision == self.with_revision.revision


class TestAddingRepositoryNameToBuildStatusPayload(BuildStatusPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.with_repository_name = cls.builder.add_repository_name(
            cls.repository_name)

    def test_immutability_after_add(self):
        assert self.with_repository_name
        assert {} == self.builder.build()

    def test_structure(self):
        assert self.repository_name == \
            self.with_repository_name.repository_name


class TestCreatingMinimalBuildStatusPayload(BuildStatusPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.payload = BuildStatusPayload() \
            .add_key(cls.key) \
            .add_state(cls.state) \
            .add_url(cls.url)
        cls.expected = json.loads(cls.resource_data(
            'BuildStatusPayload.minimal'))

    def test_minimum_viable_payload_structure_for_create(self):
        assert self.payload.validate().build() == self.expected


class TestCreatingFullBuildStatusPayload(BuildStatusPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.payload = BuildStatusPayload() \
            .add_key(cls.key) \
            .add_state(cls.state) \
            .add_url(cls.url) \
            .add_name(cls.name) \
            .add_description(cls.description)
        cls.expected = json.loads(cls.resource_data(
            'BuildStatusPayload.full'))

    def test_minimum_viable_payload_structure_for_create(self):
        assert self.payload.validate().build() == self.expected


class TestFindingBuildStatusForRepositoryCommitByKey(BuildStatusFixture):
    @httpretty.activate
    def test_response_is_a_buildstatus(self):
        httpretty.register_uri(
            httpretty.GET,
            self.resource_url(),
            content_type='application/json',
            body=self.resource_data(),
            status=200)
        response = BuildStatus.find_buildstatus_for_repository_commit_by_key(
            repository_name=self.repository_name,
            revision=self.revision,
            key=self.key,
            owner=self.owner,
            client=self.test_client)
        assert isinstance(response, BuildStatus)



# -*- coding: utf-8 -*-
from util import JsonSampleDataFixture
from test_auth import FakeAuth
from pybitbucket.bitbucket import Client

import json
from pybitbucket.bitbucket import BitbucketBase


class BitbucketFixture(JsonSampleDataFixture):
    # GIVEN: A test Bitbucket client with test credentials
    test_client = Client(FakeAuth())

    @classmethod
    def get_link_url(cls, name):
        return (
            cls.example_object()
            .data
            .get('links', {})
            .get(name, {})
            .get('href'))

    # GIVEN: The URL for the example resource
    @classmethod
    def resource_url(cls):
        return cls.get_link_url('self')


class BitbucketBaseFixture(BitbucketFixture):
    # GIVEN: Example data for a Bitbucket resource with links
    @classmethod
    def repository_data(cls):
        return cls.data_from_file('Repository.json')


class TestGettingLinksFromExampleData(BitbucketBaseFixture):
    @classmethod
    def setup_class(cls):
        data = json.loads(cls.repository_data())
        cls.links = {
            name: url
            for (name, url)
            in BitbucketBase.links_from(data)}

    def test_includes_a_link_named_self(self):
        # Much of the v2 classification relies on parsing self.
        assert self.links.get('self')

    def test_does_not_include_the_quirky_clone_link(self):
        # Clone links do not follow HAL conventions.
        # And they cannot be traversed with an HTTP client.
        assert not self.links.get('clone')

    def test_count_matches_nine(self):
        # Count of the links in the example data,
        # not including the clone links.
        assert 9 == len(list(self.links))



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""
Provides a class for manipulating User resources on Bitbucket.
"""

from pybitbucket.bitbucket import Bitbucket, BitbucketBase, Client


class User(BitbucketBase):
    id_attribute = 'username'
    resource_type = 'users'

    @staticmethod
    def is_type(data):
        return (User.has_v2_self_url(data))

    def __init__(self, data, client=Client()):
        super(User, self).__init__(data, client=client)
        # Some relationships are only available via the 1.0 API.
        # Create a "mock" UserV1 for those links.
        self.v1 = UserV1(data, client)

    @staticmethod
    def find_current_user(client=Client()):
        """
        A convenience method for finding the current user.
        In contrast to the pure hypermedia driven method on the Bitbucket
        class, this method returns a User object, instead of the
        generator.
        """
        return next(Bitbucket(client=client).userForMyself())

    @staticmethod
    def find_user_by_username(username, client=Client()):
        """
        A convenience method for finding a specific user.
        In contrast to the pure hypermedia driven method on the Bitbucket
        class, this method returns a User object, instead of the
        generator.
        """
        return next(Bitbucket(client=client).userByUsername(
            username=username))


class UserAdapter(object):
    def __init__(self, data, client=Client()):
        self.client = client
        if data.get('user') is not None:
            # A 1.0 shape has a user container.
            self.username = data['user'].get('username')
        else:
            # but when constructing from a 2.0 shape
            # then the username is a simple attribute.
            self.username = data.get('username')

    def self(self):
        return User.find_user_by_username(
            self.username,
            client=self.client)


class UserV1(BitbucketBase):
    id_attribute = 'username'
    links_json = """
{
  "_links": {
    "plan": {
      "href": "{+bitbucket_url}/1.0/users{/username}/plan"
    },
    "followers": {
      "href": "{+bitbucket_url}/1.0/users{/username}/followers"
    },
    "events": {
      "href": "{+bitbucket_url}/1.0/users{/username}/events"
    },
    "consumers": {
      "href": "{+bitbucket_url}/1.0/users{/username}/consumers"
    },
    "emails": {
      "href": "{+bitbucket_url}/1.0/users{/username}/emails"
    },
    "invitations": {
      "href": "{+bitbucket_url}/1.0/users{/username}/invitations"
    },
    "privileges": {
      "href": "{+bitbucket_url}/1.0/users{/username}/privileges"
    },
    "ssh-keys": {
      "href": "{+bitbucket_url}/1.0/users{/username}/ssh-keys"
    }
  }
}
"""

    @staticmethod
    def is_type(data):
        return (
            # Make sure there is a user structure
            (data.get('user') is not None) and
            # Categorize as 1.0 structure
            (data['user'].get('resource_uri') is not None) and
            # Categorize as user-like (user or team)
            (data['user'].get('username') is not None) and
            # Categorize as user, not team
            (data['user'].get('is_team') is False))

    def __init__(self, data, client=Client()):
        # This completely overrides the base constructor
        # because the user data is a child of the root object.
        self.data = data
        self.client = client
        if data.get('user'):
            self.__dict__.update(data['user'])
        if data.get('repositories'):
            self.repositories = [
                client.convert_to_object(r)
                for r
                in data['repositories']]
        self.v2 = UserAdapter(data, client)
        expanded_links = self.expand_link_urls(
            bitbucket_url=client.get_bitbucket_url(),
            username=self.v2.username)
        self.links = expanded_links.get('_links', {})
        self.add_remote_relationship_methods(expanded_links)


Client.bitbucket_types.add(User)
Client.bitbucket_types.add(UserV1)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""
Classes for abstracting over different forms of Bitbucket authentication.

"""
from requests.utils import default_user_agent
from requests import Session
from requests.auth import HTTPBasicAuth
from requests_oauthlib import OAuth1Session, OAuth2Session
from uritemplate import expand

from pybitbucket import metadata


class Authenticator(object):

    @staticmethod
    def user_agent_header():
        return u'{0}/{1} {2}'.format(
            metadata.package,
            metadata.version,
            default_user_agent())

    @staticmethod
    def headers(email=None, user_agent=None):
        user_agent = user_agent or Authenticator.user_agent_header()
        headers = {
            'Accept': 'application/json',
            'User-Agent': user_agent,
        }
        if email:
            headers.update({'From': email})
        return headers

    def start_http_session(self, session=None):
        session = session or Session()
        if not isinstance(session, Session):
            raise TypeError('session argument shall be of Session type')
        session.headers.update(self.headers())
        return session

    def get_username(self):
        return ""

    def who_am_i(self):
        response = self.session.get(self.who_am_i_url)
        response.raise_for_status()
        return response.json()['username']

    def __init__(self, server_base_uri=None, session=None):
        self.server_base_uri = server_base_uri or 'https://api.bitbucket.org'
        self.who_am_i_url = expand(
            '{+server_base_uri}/2.0/user',
            {'server_base_uri': self.server_base_uri})
        self.session = self.start_http_session(session)


class Anonymous(Authenticator):
    pass


class BasicAuthenticator(Authenticator):

    def start_http_session(self, session=None):
        session = session or Session()
        if not isinstance(session, Session):
            raise TypeError('session argument shall be of Session type')
        session.headers.update(self.headers(email=self.client_email))
        session.auth = HTTPBasicAuth(self.username, self.password)
        return session

    def get_username(self):
        return self.username

    def __init__(
            self,
            username,
            password,
            client_email,
            server_base_uri=None,
            session=None):
        self.username = username
        self.password = password
        self.client_email = client_email
        super(BasicAuthenticator, self).__init__(
                server_base_uri=server_base_uri,
                session=session
        )


class OAuth1Authenticator(Authenticator):
    def __init__(
            self,
            client_key,
            client_secret,
            client_email=None,
            access_token=None,
            access_token_secret=None,
            server_base_uri=None,
            session=None):

        self.client_key = client_key
        self.client_secret = client_secret
        self.client_email = client_email
        self.access_token = access_token
        self.access_token_secret = access_token_secret
        self.username = None
        super(OAuth1Authenticator, self).__init__(
                server_base_uri=server_base_uri,
                session=session
        )

    def get_username(self):
        if not self.username:
            self.username = self.who_am_i()
        return self.username

    def start_http_session(self, session=None):
        session = session or OAuth1Session(
            self.client_key,
            client_secret=self.client_secret,
            resource_owner_key=self.access_token,
            resource_owner_secret=self.access_token_secret)
        if not isinstance(session, OAuth1Session):
            raise TypeError('session argument shall be of OAuth1Session type')
        session.headers.update(self.headers(email=self.client_email))
        return session


class OAuth2Grant(object):
    def obtain_authorization(self, session, auth_uri):
        raise NotImplementedError()
        # return redirect_response


class OAuth2Authenticator(Authenticator):
    def __init__(
            self,
            client_id,
            client_secret,
            client_email,
            grant,
            redirect_uris=None,
            server_base_uri=None,
            redirect_response=None,
            client_name=None,
            client_description=None,
            auth_uri=None,
            token_uri=None,
            session=None):
        self.client_id = client_id
        self.client_secret = client_secret
        self.client_email = client_email
        self.grant = grant
        self.redirect_uris = redirect_uris
        self.client_name = client_name
        self.client_description = client_description
        self.redirect_response = redirect_response
        self.username = None
        self.server_base_uri = server_base_uri or 'https://api.bitbucket.org'
        self.auth_uri = (
            auth_uri or expand(
                '{+server_base_uri}/site/oauth2/authorize',
                {'server_base_uri': self.server_base_uri}))
        self.token_uri = (
            token_uri or expand(
                '{+server_base_uri}/site/oauth2/access_token',
                {'server_base_uri': self.server_base_uri}))
        super(OAuth2Authenticator, self).__init__(
                server_base_uri=server_base_uri,
                session=session
        )

    def start_http_session(self, session=None):
        session = session or OAuth2Session(self.client_id)
        if not isinstance(session, OAuth2Session):
            raise TypeError('session argument shall be of OAuth2Session type instead of {}'.format(type(session)))
        session.headers.update(self.headers(email=self.client_email))
        if not session.authorized:
            self.redirect_response = self.grant.obtain_authorization(
                session,
                self.auth_uri)
        session.fetch_token(
            self.token_uri,
            authorization_response=self.redirect_response)
        return session

    def get_username(self):
        if not self.username:
            self.username = self.who_am_i()
        return self.username



# -*- coding: utf-8 -*-
import json
from os import path

from pybitbucket.bitbucket import BitbucketBase


class TestV2Categorizer(object):
    @classmethod
    def setup_class(cls):
        cls.test_dir, current_file = path.split(path.abspath(__file__))

    def test_repository_categorization(self):
        example_path = path.join(
            self.test_dir,
            'Repository.json')
        with open(example_path) as f:
            example = json.load(f)
        assert BitbucketBase._has_v2_self_url(
            example, 'repositories', 'full_name')

    def test_commit_categorization(self):
        example_path = path.join(
            self.test_dir,
            'Commit.json')
        with open(example_path) as f:
            example = json.load(f)
        assert BitbucketBase._has_v2_self_url(
            example, 'commit', 'hash')



# -*- coding: utf-8 -*-
import httpretty
import json
from os import path
from test_auth import FakeAuth

from util import data_from_file

from pybitbucket.commit import Commit
from pybitbucket.comment import Comment
from pybitbucket.repository import Repository
from pybitbucket.user import User
from pybitbucket.bitbucket import Client


class TestCommit(object):
    @classmethod
    def setup_class(cls):
        cls.test_dir, current_file = path.split(path.abspath(__file__))
        cls.client = Client(FakeAuth())

    def load_example_commit(self):
        example_path = path.join(
            self.test_dir,
            'Commit.json')
        with open(example_path) as f:
            example = json.load(f)
        return Commit(example, client=self.client)

    def test_commit_string_representation(self):
        # Just tests that the __str__ method works and
        # that it does not use the default representation
        commit_str = "%s" % self.load_example_commit()
        assert not commit_str.startswith('<')
        assert not commit_str.endswith('>')
        assert commit_str.startswith('Commit hash:')

    @httpretty.activate
    def test_find_commit_by_revision(self):
        commit_hash = 'c021208234c65439f57b8244517a2b850b3ecf44'
        url = (
            self.client.get_bitbucket_url() +
            '/2.0/repositories/' +
            'teamsinspace/teamsinspace.bitbucket.org' +
            '/commit/' +
            commit_hash)
        example = data_from_file(
            self.test_dir,
            'Commit.json')
        httpretty.register_uri(
            httpretty.GET,
            url,
            content_type='application/json',
            body=example,
            status=200)
        commit = Commit.find_commit_in_repository_by_revision(
            'teamsinspace',
            'teamsinspace.bitbucket.org',
            commit_hash,
            client=self.client)
        assert isinstance(commit, Commit)
        assert commit_hash == commit.hash
        assert 'Testing with some copied html' == commit.message

    def test_commit_author(self):
        commit = self.load_example_commit()
        assert 'Daniel  Stevens <dstevens@atlassian.com>' == commit.raw_author
        assert isinstance(commit.author, User)
        assert 'dans9190' == commit.author.username
        assert 'Daniel  Stevens' == commit.author.display_name

    def test_commit_repository(self):
        commit = self.load_example_commit()
        assert isinstance(commit.repository, Repository)

    @httpretty.activate
    def test_commit_comments(self):
        commit_hash = 'c021208234c65439f57b8244517a2b850b3ecf44'
        commit = self.load_example_commit()
        url = (
            'https://' +
            'api.bitbucket.org' +
            '/2.0/repositories/' +
            'teamsinspace/teamsinspace.bitbucket.org' +
            '/commit/' +
            commit_hash +
            '/comments')
        example = data_from_file(
            self.test_dir,
            'Comment_list.json')
        httpretty.register_uri(
            httpretty.GET,
            url,
            content_type='application/json',
            body=example,
            status=200)
        assert list(commit.comments())
        assert isinstance(next(commit.comments()), Comment)

    @httpretty.activate
    def test_commit_approval(self):
        commit_hash = 'c021208234c65439f57b8244517a2b850b3ecf44'
        commit = self.load_example_commit()
        url = (
            'https://' +
            'api.bitbucket.org' +
            '/2.0/repositories/' +
            'teamsinspace/teamsinspace.bitbucket.org' +
            '/commit/' +
            commit_hash +
            '/approve')
        example = data_from_file(
            self.test_dir,
            'example_approve_commit.json')
        httpretty.register_uri(
            httpretty.POST,
            url,
            content_type='application/json',
            body=example,
            status=200)
        assert commit.approve()

        httpretty.register_uri(
            httpretty.DELETE,
            url,
            status=204)
        assert commit.unapprove()

    def test_commit_parents(self):
        commit = self.load_example_commit()
        assert list(commit.parents)
        assert isinstance(commit.parents[0], Commit)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from os import path
import sys

if sys.version_info < (3, 0):
    import io
    open = io.open


def data_from_file(directory, filename):
    """
    Deprecated. Moving to class fixtures, like JsonSampleDataFixture below.
    """
    filepath = path.join(directory, filename)
    with open(filepath, encoding='utf-8') as f:
        data = f.read()
    return data


class JsonSampleDataFixture(object):
    # GIVEN: a class under test
    class_under_test = 'Bitbucket'

    # GIVEN: a utility for deciding where test data lives
    @classmethod
    def test_dir(cls):
        this_dir, this_file = path.split(path.abspath(__file__))
        return this_dir

    # GIVEN: a utility for loading json files
    @classmethod
    def data_from_file(cls, filename, directory=None):
        if (directory is None):
            directory = cls.test_dir()
        filepath = path.join(directory, filename)
        with open(filepath, encoding='utf-8') as f:
            data = f.read()
        return data

    # GIVEN: Example data for a resource
    @classmethod
    def resource_data(cls, name=None):
        if name is None:
            name = cls.class_under_test
        file_name = '{}.json'.format(name)
        return cls.data_from_file(file_name)

    # GIVEN: Example data for a set of resources
    @classmethod
    def resource_list_data(cls, name=None):
        if name is None:
            name = cls.class_under_test
        file_name = '{}_list.json'.format(name)
        return cls.data_from_file(file_name)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""
Defines the Consumer resource and registers the type with the Client.

Classes:
- PermissionScope: enumerates the possible scopes for an OAuth consumer.
- ConsumerPayload: encapsulates payload for creating
    and modifying consumers.
- Consumer: represents an OAuth consumer.
"""

from uritemplate import expand
from voluptuous import Schema, Required, Optional, In

from pybitbucket.bitbucket import BitbucketBase, Client, PayloadBuilder, Enum


class PermissionScope(Enum):
    EMAIL = 'email'
    ACCOUNT_READ = 'account'
    ACCOUNT_WRITE = 'account:write'
    TEAM_READ = 'team'
    TEAM_WRITE = 'team:write'
    REPOSITORY_READ = 'repository'
    REPOSITORY_WRITE = 'repository:write'
    REPOSITORY_ADMIN = 'repository:admin'
    PULLREQUEST_READ = 'pullrequest'
    PULLREQUEST_WRITE = 'pullrequest:write'
    ISSUE_READ = 'issue'
    ISSUE_WRITE = 'issue:write'
    WIKI = 'wiki'
    SNIPPET_READ = 'snippet'
    SNIPPET_WRITE = 'snippet:write'
    WEBHOOK = 'webhook'


class ConsumerPayload(PayloadBuilder):
    """
    A builder object to help create payloads
    for creating and updating consumers.
    """

    schema = Schema({
        Required('name'): str,
        Optional('description'): str,
        Optional('url'): str,
        Optional('key'): str,
        # Undocumented attributes
        Optional('scopes'): [In(PermissionScope)],
        Optional('secret'): str,
        Optional('callback_url'): str,
        Optional('id'): int,
    })

    def __init__(
            self,
            payload=None,
            consumer_id=None):
        super(self.__class__, self).__init__(payload=payload)
        self._consumer_id = consumer_id

    @property
    def consumer_id(self):
        return self._consumer_id

    def add_consumer_id(self, consumer_id):
        new = self._payload.copy()
        new['id'] = consumer_id
        return ConsumerPayload(
            payload=new,
            consumer_id=consumer_id)

    def add_name(self, name):
        new = self._payload.copy()
        new['name'] = name
        return ConsumerPayload(
            payload=new,
            consumer_id=self.consumer_id)

    def add_description(self, description):
        new = self._payload.copy()
        new['description'] = description
        return ConsumerPayload(
            payload=new,
            consumer_id=self.consumer_id)

    def add_url(self, url):
        new = self._payload.copy()
        new['url'] = url
        return ConsumerPayload(
            payload=new,
            consumer_id=self.consumer_id)

    def add_key(self, key):
        new = self._payload.copy()
        new['key'] = key
        return ConsumerPayload(
            payload=new,
            consumer_id=self.consumer_id)

    def add_scope(self, scope):
        new = self._payload.copy()
        new_scopes = self._payload.get('scopes', [])
        if scope not in new_scopes:
            new_scopes.append(scope)
        new['scopes'] = new_scopes
        return ConsumerPayload(
            payload=new,
            consumer_id=self.consumer_id)

    def add_scopes(self, scopes):
        new = self._payload.copy()
        new_scopes = self._payload.get('scopes', [])
        for scope in scopes:
            if scope not in new_scopes:
                new_scopes.append(scope)
        new['scopes'] = new_scopes
        return ConsumerPayload(
            payload=new,
            consumer_id=self.consumer_id)

    def add_secret(self, secret):
        new = self._payload.copy()
        new['secret'] = secret
        return ConsumerPayload(
            payload=new,
            consumer_id=self.consumer_id)

    def add_callback_url(self, callback_url):
        new = self._payload.copy()
        new['callback_url'] = callback_url
        return ConsumerPayload(
            payload=new,
            consumer_id=self.consumer_id)


class Consumer(BitbucketBase):
    id_attribute = 'id'
    links_json = """
{
  "_links": {
    "self": {
      "href": "{+bitbucket_url}/1.0/users{/username}/consumers{/consumer_id}"
    },
    "owner": {
      "href": "{+bitbucket_url}/1.0/users{/username}"
    },
    "consumers": {
      "href": "{+bitbucket_url}/1.0/users{/username}/consumers"
    },
    "create": {
      "href": "{+bitbucket_url}/1.0/users{/username}/consumers"
    }
  }
}
"""

    @staticmethod
    def is_type(data):
        return (
            (data.get('id') is not None) and
            (data.get('name') is not None) and
            (data.get('secret') is not None) and
            (data.get('key') is not None))

    def __init__(self, data={}, client=None):
        client = client or Client()
        super(Consumer, self).__init__(data, client=client)
        expanded_links = self.expand_link_urls(
            bitbucket_url=client.get_bitbucket_url(),
            username=client.get_username(),
            consumer_id=data.get('id'))
        self.links = expanded_links.get('_links', {})
        self.templates = self.extract_templates_from_json()
        self.add_remote_relationship_methods(expanded_links)

    @classmethod
    def create(
            cls,
            payload,
            client=None):
        """Create a new consumer.

        :param payload: the options for creating the new consumer
        :type payload: ConsumerPayload
        :param client: the configured connection to Bitbucket.
            If not provided, assumes an Anonymous connection.
        :type client: bitbucket.Client
        :returns: the new consumer object.
        :rtype: Consumer
        :raises: ValueError
        """
        client = client or Client()
        owner = client.get_username()
        if not owner:
            raise ValueError('owner is required')
        data = payload.validate().build()
        templates = cls.extract_templates_from_json()
        api_url = expand(
            templates['create'], {
                'bitbucket_url': client.get_bitbucket_url(),
                'username': owner
            })
        # Note: This Bitbucket API expects a urlencoded-form, not json.
        # Hence, use `data` instead of `json`.
        return cls.post(api_url, data=data, client=client)

    def update(
            self,
            payload):
        """Update current consumer.

        :param payload: the options for updating the new consumer
        :type payload: ConsumerPayload
        :returns: the new consumer object.
        :rtype: Consumer
        :raises: ValueError
        """
        # Note: The Bitbucket API expects a urlencoded-form, not json.
        # Hence, use `data` instead of `json`.
        return self.put(data=payload.validate().build())

    @staticmethod
    def find_consumers(client=Client()):
        """
        Find consumers for the authenticated user.
        The method is a generator Consumer objects.
        """
        url = expand(
            Consumer.get_link_template('consumers'), {
                'bitbucket_url': client.get_bitbucket_url(),
                'username': client.get_username()
            })
        return client.remote_relationship(url)

    @staticmethod
    def find_consumer_by_id(consumer_id, client=Client()):
        """
        Finding a specific consumer by id for the authenticated user.
        """
        url = expand(
            Consumer.get_link_template('self'), {
                'bitbucket_url': client.get_bitbucket_url(),
                'username': client.get_username(),
                'consumer_id': consumer_id,
            })
        return next(client.remote_relationship(url))


Client.bitbucket_types.add(Consumer)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""
Defines the Commit resource and registers the type with the Client.

Classes:
- Commit: represents a Git or Hg commit
"""
from functools import partial
from uritemplate import expand

from pybitbucket.bitbucket import BitbucketBase, Client


class Commit(BitbucketBase):
    id_attribute = 'hash'
    resource_type = 'commit'

    @staticmethod
    def is_type(data):
        return (Commit.has_v2_self_url(data))

    # Must override base constructor to account for approve and unapprove
    def __init__(self, data, client=Client()):
        super(Commit, self).__init__(data, client=client)
        # approve and unapprove are just different verbs for same url
        if data.get('links', {}).get('approve', {}).get('href', {}):
            url = data['links']['approve']['href']
            setattr(self, 'approve', partial(
                self.post_approval, template=url))
            setattr(self, 'unapprove', partial(
                self.delete_approval, template=url))

    @staticmethod
    def find_commit_in_repository_by_revision(
            username,
            repository_name,
            revision,
            client=Client()):
        template = (
            '{+bitbucket_url}' +
            '/2.0/repositories/{username}/{repository_name}' +
            '/commit/{revision}')
        url = expand(
            template,
            {
                'bitbucket_url': client.get_bitbucket_url(),
                'username': username,
                'repository_name': repository_name,
                'revision': revision
            })
        response = client.session.get(url)
        if 404 == response.status_code:
            return
        Client.expect_ok(response)
        return Commit(response.json(), client=client)

    @staticmethod
    def find_commit_in_repository_full_name_by_revision(
            repository_full_name,
            revision,
            client=Client()):
        if '/' not in repository_full_name:
            raise NameError(
                "Repository full name must be in the form: username/name")
        username, repository_name = repository_full_name.split('/')
        return Commit.find_commit_in_repository_by_revision(
            username,
            repository_name,
            revision,
            client=client)

    @staticmethod
    def find_commits_in_repository(
            username,
            repository_name,
            branch=None,
            include=None,
            exclude=None,
            client=Client()):
        include = include or []
        exclude = exclude or []
        template = (
            '{+bitbucket_url}' +
            '/2.0/repositories/{username}/{repository_name}' +
            '/commits{/branch}{?include*,exclude*}')
        url = expand(
            template,
            {
                'bitbucket_url': client.get_bitbucket_url(),
                'username': username,
                'repository_name': repository_name,
                'branch': branch,
                'include': include,
                'exclude': exclude
            })
        for commit in client.remote_relationship(url):
            yield commit

    @staticmethod
    def find_commits_in_repository_full_name(
            repository_full_name,
            branch=None,
            include=None,
            exclude=None,
            client=Client()):
        include = include or []
        exclude = exclude or []
        if '/' not in repository_full_name:
            raise NameError(
                "Repository full name must be in the form: username/name")
        username, repository_name = repository_full_name.split('/')
        yield Commit.find_commits_in_repository(
            username,
            repository_name,
            branch=branch,
            include=include,
            exclude=exclude,
            client=client)


Client.bitbucket_types.add(Commit)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function

# Copyright (c) 2014 Sean Fisk and Benjamin Schwarze
#
# Generated by python-project-template:
# https://github.com/seanfisk/python-project-template
#
# Licensed under the MIT License:
# http://opensource.org/licenses/MIT
#

import os
import sys
import imp
import subprocess

from pybitbucket import metadata

# Python 2.6 subprocess.check_output compatibility. Thanks Greg Hewgill!
if 'check_output' not in dir(subprocess):
    def check_output(cmd_args, *args, **kwargs):
        proc = subprocess.Popen(
            cmd_args, *args,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kwargs)
        out, err = proc.communicate()
        if proc.returncode != 0:
            raise subprocess.CalledProcessError(args)
        return out
    subprocess.check_output = check_output

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
from distutils import spawn

try:
    import colorama
    colorama.init()  # Initialize colorama on Windows
except ImportError:
    # Don't require colorama just for running paver tasks. This allows us to
    # run `paver install' without requiring the user to first have colorama
    # installed.
    pass

# Add the current directory to the module search path.
sys.path.append('.')

# Constants
CODE_DIRECTORY = 'pybitbucket'
DOCS_DIRECTORY = 'docs'
TESTS_DIRECTORY = 'tests'
PYTEST_FLAGS = ['--doctest-modules']
TEST_REPORT_FILE = 'TESTS-TestSuites.xml'
PYTEST_FLAGS = ['--doctest-modules', '--junitxml=' + TEST_REPORT_FILE]


# Import metadata. Normally this would just be:
#
#     from pybitbucket import metadata
#
# However, when we do this, we also import `pybitbucket/__init__.py'. If this
# imports names from some other modules and these modules have third-party
# dependencies that need installing (which happens after this file is run), the
# script will crash. What we do instead is to load the metadata module by path
# instead, effectively side-stepping the dependency problem. Please make sure
# metadata has no dependencies, otherwise they will need to be added to
# the setup_requires keyword.
metadata = imp.load_source(
    'metadata',
    os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        CODE_DIRECTORY,
        'metadata.py'))


# Miscellaneous helper functions

def get_project_files():
    """Retrieve a list of project files, ignoring hidden files.

    :return: sorted list of project files
    :rtype: :class:`list`
    """
    if is_git_project() and has_git():
        return get_git_project_files()

    project_files = []
    for top, subdirs, files in os.walk('.', topdown=True):
        subdirs[:] = [d for d in subdirs
            if not d.startswith('.')]
        [project_files.append(os.path.join(top, f))
            for f in files
            if not f.startswith('.')]
    return project_files


def is_git_project():
    return os.path.isdir('.git')


def has_git():
    return bool(spawn.find_executable("git"))


def get_git_project_files():
    """Retrieve a list of all non-ignored files, including untracked files,
    excluding deleted files.

    :return: sorted list of git project files
    :rtype: :class:`list`
    """
    cached_and_untracked_files = git_ls_files(
        '--cached',  # All files cached in the index
        '--others',  # Untracked files
        # Exclude untracked files that would be excluded by .gitignore, etc.
        '--exclude-standard')
    uncommitted_deleted_files = git_ls_files('--deleted')

    # Since sorting of files in a set is arbitrary, return a sorted list to
    # provide a well-defined order to tools like flake8, etc.
    return sorted(cached_and_untracked_files - uncommitted_deleted_files)


def safe_str(obj):
    try:
        return str(obj)
    except UnicodeDecodeError:
        # obj is byte string
        text = obj.decode()
        return str(text)


def git_ls_files(*cmd_args):
    """Run ``git ls-files`` in the top-level project directory. Arguments go
    directly to execution call.

    :return: set of file names
    :rtype: :class:`set`
    """
    cmd = ['git', 'ls-files']
    cmd.extend(cmd_args)
    files = [safe_str(f) for f in subprocess.check_output(cmd).splitlines()]
    return set(files)


def print_success_message(message):
    """Print a message indicating success in green color to STDOUT.

    :param message: the message to print
    :type message: :class:`str`
    """
    try:
        import colorama
        print(colorama.Fore.GREEN + message + colorama.Fore.RESET)
    except ImportError:
        print(message)


def print_failure_message(message):
    """Print a message indicating failure in red color to STDERR.

    :param message: the message to print
    :type message: :class:`str`
    """
    try:
        import colorama
        print(colorama.Fore.RED + message + colorama.Fore.RESET,
              file=sys.stderr)
    except ImportError:
        print(message, file=sys.stderr)


def read(filename):
    """Return the contents of a file.

    :param filename: file path
    :type filename: :class:`str`
    :return: the file's content
    :rtype: :class:`str`
    """
    with open(os.path.join(os.path.dirname(__file__), filename)) as f:
        return f.read()


def _lint():
    """Run lint and return an exit code."""
    # Flake8 doesn't have an easy way to run checks using a Python function, so
    # just fork off another process to do it.

    # Python 3 compat:
    # - The result of subprocess call outputs are byte strings, meaning we need
    #   to pass a byte string to endswith.
    project_python_files = [filename for filename in get_project_files()
                            if filename.endswith('.py')]
    retcode = subprocess.call(
        ['flake8', '--max-complexity=10'] + project_python_files)
    if retcode == 0:
        print_success_message('No style errors')
    return retcode


def _test():
    """Run the unit tests.

    :return: exit code
    """
    # Make sure to import pytest in this function. For the reason, see here:
    # <http://pytest.org/latest/goodpractises.html#integration-with-setuptools-test-commands>  # NOPEP8
    import pytest
    # This runs the unit tests.
    # It also runs doctest, but only on the modules in TESTS_DIRECTORY.
    return pytest.main(PYTEST_FLAGS + [TESTS_DIRECTORY])


def _test_all():
    """Run lint and tests.

    :return: exit code
    """
    return _lint() + _test()


# The following code is to allow tests to be run with `python setup.py test'.
# The main reason to make this possible is to allow tests to be run as part of
# Setuptools' automatic run of 2to3 on the source code. The recommended way to
# run tests is still `paver test_all'.
# See <http://pythonhosted.org/setuptools/python3.html>
# Code based on <http://pytest.org/latest/goodpractises.html#integration-with-setuptools-test-commands>  # NOPEP8
class TestAllCommand(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        # These are fake, and just set to appease distutils and setuptools.
        self.test_suite = True
        self.test_args = []

    def run_tests(self):
        raise SystemExit(_test_all())


# define install_requires for specific Python versions
python_version_specific_requires = []

# as of Python >= 2.7 and >= 3.2, the argparse module is maintained within
# the Python standard library, otherwise we install it as a separate package
if sys.version_info < (2, 7) or (3, 0) <= sys.version_info < (3, 3):
    python_version_specific_requires.append('argparse')

# enum package has been introduced to python standard library in python 3.4
if sys.version_info < (3, 4):
    python_version_specific_requires.append('enum34')


# See here for more options:
# <http://pythonhosted.org/setuptools/setuptools.html>
setup_dict = dict(
    name=metadata.package,
    version=metadata.version,
    author=metadata.authors[0],
    author_email=metadata.emails[0],
    maintainer=metadata.authors[0],
    maintainer_email=metadata.emails[0],
    url=metadata.url,
    bugtrack_url=metadata.bugtrack_url,
    description=metadata.description,
    long_description=read('README.rst'),
    # Find a list of classifiers here:
    # <http://pypi.python.org/pypi?%3Aaction=list_classifiers>
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'License :: DFSG approved',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Version Control',
    ],
    packages=find_packages(exclude=(TESTS_DIRECTORY,)),
    install_requires=[
        'six >= 1.10, < 2',
        'future >= 0.16, < 1',
        'requests >= 2.12, < 3',
        'oauthlib >= 2.0, < 3',
        'requests_oauthlib >= 0.7, < 1',
        'uritemplate >= 0.6, < 1',
        'simplejson >= 3.10, < 4',
        'voluptuous >= 0.9, < 1'
        # your module dependencies
    ] + python_version_specific_requires,
    # Allow tests to be run with `python setup.py test'.
    tests_require=[
        'pytest >= 2.7, < 3',
        'mock >= 1.0, < 2',
        'flake8 >= 2.4, < 3',
        'httpretty >= 0.8, < 1',
    ],
    cmdclass={'test': TestAllCommand},
    zip_safe=False,  # don't use eggs
    entry_points={
        'console_scripts': [
            'pybitbucket_cli = pybitbucket.main:entry_point'
        ]
    }
)


def main():
    setup(**setup_dict)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

entrypoints_json = """
{
  "_links": {
    "self": {
      "href": "https://api.bitbucket.org/"
    },
    "userForMyself": {
      "href": "https://api.bitbucket.org/2.0/user"
    },
    "userByUsername": {
      "href": "https://api.bitbucket.org/2.0/users{/username}"
    },
    "teamsForRole": {
      "href": "https://api.bitbucket.org/2.0/teams{?role}"
    },
    "teamByUsername": {
      "href": "https://api.bitbucket.org/2.0/teams{/username}"
    },
    "repositoriesThatArePublic": {
      "href": "https://api.bitbucket.org/2.0/repositories"
    },
    "repositoriesByOwnerAndRole": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner}{?role}"
    },
    "repositoryByOwnerAndRepositoryName": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}"
    },
    "repositoryWatchers": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/watchers"
    },
    "repositoryForks": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/forks"
    },
    "repositoryBranchRestrictions": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/branch-restrictions"
    },
    "repositoryBranchRestrictionByRestrictionId": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/branch-restrictions{/restriction_id}"
    },
    "repositoryCommits": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/commits{/branch}{?include*,exclude*}"
    },
    "repositoryCommitByRevision": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/commit{/revision}"
    },
    "repositoryCommitComments": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/commit{/revision}/comments"
    },
    "repositoryCommitCommentByCommentId": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/commit{/revision}/comments{/comment_id}"
    },
    "repositoryCommitBuildStatuses": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/commit{/revision}/statuses/build"
    },
    "repositoryCommitBuildStatusByKey": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/commit{/revision}/statuses/build{/key}"
    },
    "repositoryRefs": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/refs"
    },
    "repositoryTags": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/refs/tags"
    },
    "repositoryTagByName": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/refs/tags{/ref_name}"
    },
    "repositoryBranches": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/refs/branches"
    },
    "repositoryBranchByName": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/refs/branches{/ref_name}"
    },
    "repositoryPullRequestsInState": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/pullrequests{?state}"
    },
    "repositoryPullRequestActivitiesForWholeRepository": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/pullrequests/activity"
    },
    "repositoryPullRequestByPullRequestId": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/pullrequests{/pullrequest_id}"
    },
    "repositoryPullRequestActivitiesForPullRequest": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/pullrequests{/pullrequest_id}/activity"
    },
    "repositoryPullRequestCommits": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/pullrequests{/pullrequest_id}/commits"
    },
    "repositoryPullRequestDiff": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/pullrequests{/pullrequest_id}/diff"
    },
    "repositoryPullRequestComments": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/pullrequests{/pullrequest_id}/comments"
    },
    "repositoryPullRequestCommentsByCommentId": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/pullrequests{/pullrequest_id}/comments{/comment_id}"
    },
    "repositoryHooks": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/hooks"
    },
    "repositoryHookById": {
      "href": "https://api.bitbucket.org/2.0/repositories{/owner,repository_name}/hooks{/uuid}"
    },
    "snippetsThatArePublic": {
      "href": "https://api.bitbucket.org/2.0/snippets"
    },
    "snippetsForRole": {
      "href": "https://api.bitbucket.org/2.0/snippets{?role}"
    },
    "snippetByOwner": {
      "href": "https://api.bitbucket.org/2.0/snippets{/owner}"
    },
    "snippetByOwnerAndSnippetId": {
      "href": "https://api.bitbucket.org/2.0/snippets{/owner,snippet_id}"
    },
    "snippetCommentByCommentId": {
      "href": "https://api.bitbucket.org/2.0/snippets{/owner,snippet_id}/comments{/comment_id}"
    },
    "privilegesForOwner": {
      "href": "https://api.bitbucket.org/1.0/privileges{/owner}"
    },
    "privilegesForRepository": {
      "href": "https://api.bitbucket.org/1.0/privileges{/owner,repository_name}"
    },
    "privilegesForRepositoryAndUsername": {
      "href": "https://api.bitbucket.org/1.0/privileges{/owner,repository_name,username}"
    },
    "groupsForFilter": {
      "href": "https://api.bitbucket.org/1.0/groups{?filter*}"
    },
    "groupsForOwner": {
      "href": "https://api.bitbucket.org/1.0/groups{/owner}"
    },
    "groupForOwnerAndGroupId": {
      "href": "https://api.bitbucket.org/1.0/groups{/owner,group_id}"
    },
    "groupMembers": {
      "href": "https://api.bitbucket.org/1.0/groups{/owner,group_id}/members"
    },
    "groupPrivilegesForRepositoryOwner": {
      "href": "https://api.bitbucket.org/1.0/group-privileges{/repository_owner}"
    },
    "groupPrivilegesForRepository": {
      "href": "https://api.bitbucket.org/1.0/group-privileges{/repository_owner,repository_id}"
    },
    "groupPrivilegesForRepositoryPrivilegesGroup": {
      "href": "https://api.bitbucket.org/1.0/group-privileges{/repository_owner,group_owner,group_id}"
    },
    "groupForRepositoryAndGroup": {
      "href": "https://api.bitbucket.org/1.0/group-privileges{/repository_owner,repository_id,group_owner,group_id}"
    }
  }
}
"""



# -*- coding: utf-8 -*-
from test_bitbucketbase import BitbucketFixture
import json

import httpretty
from uritemplate import expand, variables
from pybitbucket.consumer import Consumer, ConsumerPayload, PermissionScope
from voluptuous import MultipleInvalid


class ConsumerFixture(BitbucketFixture):
    # GIVEN: a class under test
    class_under_test = 'Consumer'

    # GIVEN: An example object created from example data
    @classmethod
    def example_object(cls):
        return Consumer(
            json.loads(cls.resource_data()),
            client=cls.test_client)

    # GIVEN: The URL for the example resource
    @classmethod
    def resource_url(cls):
        return expand(
            Consumer(client=cls.test_client).templates['self'], {
                'bitbucket_url': cls.test_client.get_bitbucket_url(),
                'username': cls.test_client.get_username(),
                'consumer_id': cls.consumer_id
            })

    # GIVEN: The URL for listing example resources
    @classmethod
    def resource_list_url(cls):
        return expand(
            Consumer(client=cls.test_client).templates['self'], {
                'bitbucket_url': cls.test_client.get_bitbucket_url(),
                'username': cls.test_client.get_username(),
            })

    # GIVEN: Example inputs for creating a new consumer
    name = 'code-registered-client'
    description = 'Some code added this consumer via the REST API.'
    consumer_url = 'http://example.com/'
    callback_url = 'http://localhost'
    scopes = [PermissionScope.REPOSITORY_READ, PermissionScope.EMAIL]
    consumer_id = 302126
    consumer_key = '6qErCQRtk2nmVHuemM'
    shared_secret = '96ztLLDzvrT5zmchsdZZLKCkTMkUkb4W'


class ConsumerPayloadFixture(ConsumerFixture):
    builder = ConsumerPayload()

    # GIVEN: a class under test
    class_under_test = 'ConsumerPayload'

    # GIVEN: An example object created from example data
    @classmethod
    def example_object(cls):
        return ConsumerPayload(json.loads(cls.resource_data()))


class TestGettingTheStringRepresentation(ConsumerFixture):
    @classmethod
    def setup_class(cls):
        cls.branchrestriction_str = str(cls.example_object())

    def test_string_is_not_the_default_format(self):
        assert not self.branchrestriction_str.startswith('<')
        assert not self.branchrestriction_str.endswith('>')

    def test_string_has_the_class_name_and_id_attribute(self):
        assert self.branchrestriction_str.startswith('Consumer id:')


class TestCheckingTheExampleData(ConsumerFixture):
    @classmethod
    def setup_class(cls):
        cls.data = json.loads(cls.resource_data())

    def test_passes_the_type_check_for_consumer(self):
        assert Consumer.is_type(self.data)


class TestGettingLinks(ConsumerFixture):
    @classmethod
    def setup_class(cls):
        cls._self = Consumer.get_link_template('self')
        cls._owner = Consumer.get_link_template('owner')
        cls._consumers = Consumer.get_link_template('consumers')

    def test_self_has_an_id(self):
        assert 'consumer_id' in variables(self._self)

    def test_owner_is_identified_by_username(self):
        assert 'username' in variables(self._owner)

    def test_consumers_only_needs_bitbucket_url_and_username(self):
        assert set(['username', 'bitbucket_url']) == \
            variables(self._consumers)


class TestCreatingConsumerPayload(ConsumerPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.payload = ConsumerPayload() \
            .add_name(cls.name)

    def test_payload_structure(self):
        assert self.payload.validate().build() == {
                "name": self.name,
            }


class TestCreatingPayloadWithInvalidPermissionScope(ConsumerFixture):
    @classmethod
    def setup_class(cls):
        cls.builder = ConsumerPayload() \
            .add_name(cls.name)

    def test_raising_exception_for_not_an_array(self):
        try:
            self.builder = self.builder.add_scopes('invalid')
            self.builder = self.builder.validate()
            assert False
        except Exception as e:
            assert isinstance(e, MultipleInvalid)

    def test_raising_exception_for_invalid_permission_scope_name(self):
        try:
            self.builder = self.builder.add_scope('invalid')
            self.builder = self.builder.validate()
            assert False
        except Exception as e:
            assert isinstance(e, MultipleInvalid)


class TestDeletingConsumer(ConsumerFixture):
    @httpretty.activate
    def test_response_is_not_an_exception(self):
        httpretty.register_uri(
            httpretty.DELETE,
            self.resource_url(),
            status=204)
        result = self.example_object().delete()
        assert result is None


class TestCreatingPayload(ConsumerFixture):
    @classmethod
    def setup_class(cls):
        cls.payload = ConsumerPayload() \
            .add_name(cls.name) \
            .add_description(cls.description) \
            .add_url(cls.consumer_url) \
            .add_callback_url(cls.callback_url) \
            .add_scopes(cls.scopes) \
            .validate() \
            .build()

    def test_payload_structure_has_2_scopes(self):
        assert 2 == len(self.payload.get('scopes'))


class TestCreatingNewConsumer(ConsumerFixture):
    @classmethod
    def setup_class(cls):
        cls.url = expand(
            Consumer(client=cls.test_client).templates['create'], {
                'bitbucket_url': cls.test_client.get_bitbucket_url(),
                'username': cls.test_client.get_username(),
            })

    @httpretty.activate
    def test_response_is_a_consumer(self):
        httpretty.register_uri(
            httpretty.POST,
            self.url,
            content_type='application/json',
            body=self.resource_data(),
            status=200)
        payload = ConsumerPayload() \
            .add_name(self.name) \
            .add_description(self.description) \
            .add_url(self.consumer_url) \
            .add_callback_url(self.callback_url) \
            .add_scopes(self.scopes)
        response = Consumer.create(payload, client=self.test_client)
        assert 'application/x-www-form-urlencoded' == \
            httpretty.last_request().headers.get('Content-Type')
        assert 2 == \
            len(httpretty.last_request().parsed_body.get('scopes'))
        assert isinstance(response, Consumer)


class TestGettingExpandedSelfUrl(ConsumerFixture):
    def test_url_has_id(self):
        assert (
            'https://staging.bitbucket.org/api' +
            '/1.0/users/' +
            'pybitbucket' +
            '/consumers/' +
            str(self.consumer_id)) == \
            self.example_object().links['self']['href']


class TestUpdatingConsumer(ConsumerFixture):
    new_url = 'http://www.example.com'

    @httpretty.activate
    def test_response_is_a_consumer(self):
        httpretty.register_uri(
            httpretty.PUT,
            self.resource_url(),
            content_type='application/json',
            body=self.resource_data(),
            status=200)
        payload = ConsumerPayload() \
            .add_name(self.name) \
            .add_callback_url(self.callback_url)
        response = self.example_object().update(payload=payload)
        assert 'application/x-www-form-urlencoded' == \
            httpretty.last_request().headers.get('Content-Type')
        assert self.callback_url == \
            httpretty.last_request().parsed_body['callback_url'][0]
        assert isinstance(response, Consumer)


class TestFindingConsumerById(ConsumerFixture):
    @httpretty.activate
    def test_response_is_a_consumer(self):
        httpretty.register_uri(
            httpretty.GET,
            self.resource_url(),
            content_type='application/json',
            body=self.resource_data(),
            status=200)
        response = \
            Consumer.find_consumer_by_id(
                consumer_id=self.consumer_id,
                client=self.test_client)
        assert isinstance(response, Consumer)


class TestFindingConsumers(ConsumerFixture):
    @httpretty.activate
    def test_response_is_a_consumer_generator(self):
        httpretty.register_uri(
            httpretty.GET,
            self.resource_list_url(),
            content_type='application/json',
            body=self.resource_list_data(),
            status=200)
        response = Consumer.find_consumers(client=self.test_client)
        assert isinstance(next(response), Consumer)


class TestCreatingDefaultConsumerPayload(ConsumerPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.data = cls.builder.build()

    def test_default_payload_is_empty_dictionary(self):
        assert {} == self.data

    def test_default_is_invalid(self):
        try:
            self.builder.validate()
            assert False
        except Exception as e:
            assert isinstance(e, MultipleInvalid)


class TestCreatingMinimalConsumerPayload(ConsumerPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.payload = ConsumerPayload() \
            .add_name(cls.name)
        cls.expected = json.loads(cls.resource_data(
            'ConsumerPayload.minimal'))

    def test_minimum_viable_payload_structure_for_create(self):
        assert self.payload.validate().build() == self.expected


class TestCreatingFullConsumerPayload(ConsumerPayloadFixture):
    @classmethod
    def setup_class(cls):
        cls.payload = ConsumerPayload() \
            .add_name(cls.name) \
            .add_description(cls.description) \
            .add_url(cls.consumer_url) \
            .add_key(cls.consumer_key) \
            .add_secret(cls.shared_secret) \
            .add_callback_url(cls.callback_url) \
            .add_consumer_id(cls.consumer_id)
        cls.expected = json.loads(cls.resource_data(
            'ConsumerPayload.full'))

    def test_full_payload_structure(self):
        assert self.payload.validate().build() == self.expected



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""
Provides classes for manipulating Repository resources on Bitbucket.

Classes:
- RepositoryRole: enumerates the roles a user can have on a repository
- RepositoryForkPolicy: enumerates the forking policies on a repository
- RepositoryType: enumerates the SCM types for Bitbucket repositories
- RepositoryPayload: encapsulates payload for creating
    and modifying repositories
- Repository: represents a repository
- RepositoryAdapter: a bridge between 1.0 and 2.0 API representations
- RepositoryV1: represents a repository in the 1.0 API
"""
from uritemplate import expand
from voluptuous import Schema, Required, Optional, In

from pybitbucket.bitbucket import (
    Bitbucket, BitbucketBase, Client, PayloadBuilder, RepositoryType, Enum)
from pybitbucket.user import User


class RepositoryRole(Enum):
    OWNER = 'owner'
    ADMIN = 'admin'
    CONTRIBUTOR = 'contributor'
    MEMBER = 'member'


class RepositoryForkPolicy(Enum):
    ALLOW_FORKS = 'allow_forks'
    NO_PUBLIC_FORKS = 'no_public_forks'
    NO_FORKS = 'no_forks'


class RepositoryPayload(PayloadBuilder):
    """
    A builder object to help create payloads
    for creating and updating repositories.
    """

    schema = Schema({
        Optional('scm'): In(RepositoryType),
        Optional('name'): str,
        Required('is_private'): bool,
        Optional('description'): str,
        Required('fork_policy'): In(RepositoryForkPolicy),
        Optional('language'): str,
        Optional('has_issues'): bool,
        Optional('has_wiki'): bool
    })

    def __init__(self, payload=None, owner=None):
        super(RepositoryPayload, self).__init__(payload=payload)
        self._owner = owner

    @property
    def owner(self):
        return self._owner

    @property
    def name(self):
        return self._payload.get('name')

    def add_owner(self, owner):
        return RepositoryPayload(
            payload=self._payload.copy(),
            owner=owner)

    def add_name(self, name):
        new = self._payload.copy()
        new['name'] = name
        return RepositoryPayload(
            payload=new,
            owner=self._owner)

    def add_is_private(self, is_private):
        new = self._payload.copy()
        new['is_private'] = is_private
        return RepositoryPayload(
            payload=new,
            owner=self._owner)

    def add_fork_policy(self, policy):
        new = self._payload.copy()
        new['fork_policy'] = policy
        return RepositoryPayload(
            payload=new,
            owner=self._owner)

    def add_scm(self, scm):
        new = self._payload.copy()
        new['scm'] = scm
        return RepositoryPayload(
            payload=new,
            owner=self._owner)

    def add_description(self, description):
        new = self._payload.copy()
        new['description'] = description
        return RepositoryPayload(
            payload=new,
            owner=self._owner)

    def add_language(self, language):
        new = self._payload.copy()
        new['language'] = language
        return RepositoryPayload(
            payload=new,
            owner=self._owner)

    def add_has_wiki(self, has_wiki):
        new = self._payload.copy()
        new['has_wiki'] = has_wiki
        return RepositoryPayload(
            payload=new,
            owner=self._owner)

    def add_has_issues(self, has_isues):
        new = self._payload.copy()
        new['has_issues'] = has_isues
        return RepositoryPayload(
            payload=new,
            owner=self._owner)


class RepositoryForkPayload(RepositoryPayload):
    schema = Schema({
        Required('name'): str,
        Optional('scm'): In(RepositoryType),
        Optional('is_private'): bool,
        Optional('description'): str,
        Optional('language'): str,
        Optional('fork_policy'): In(RepositoryForkPolicy),
    })


class Repository(BitbucketBase):
    """Represents a repository."""

    id_attribute = 'full_name'
    resource_type = 'repositories'
    templates = {
        'create': (
            '{+bitbucket_url}' +
            '/2.0/repositories' +
            '{/owner,repository_name}'),
        'fork': (
            '{+bitbucket_url}' +
            '/1.0/repositories' +
            '{/owner,repository_name}' +
            '/fork')
    }

    @staticmethod
    def is_type(data):
        return Repository.has_v2_self_url(data)

    def __init__(self, data, client=None):
        client = client or Client()
        super(Repository, self).__init__(data, client=client)
        if data.get('links', {}).get('clone'):
            self.clone = {
                clone_method['name']: clone_method['href']
                for clone_method
                in data['links']['clone']}
        # Some relationships are only available via the 1.0 API.
        # Create a "mock" RepositoryV1 for those links.
        self.v1 = RepositoryV1(data, client)

    @classmethod
    def create(
            cls,
            payload,
            repository_name=None,
            owner=None,
            client=None):
        """Create a new repository.

        :param payload: the options for creating the new repository
        :type payload: RepositoryPayload
        :param repository_name: name of the repository,
            also known as repo_slug. Optional, if provided in the payload.
        :type repository_name: str
        :param owner: the owner of the repository.
            If not provided, assumes the current user.
        :type owner: str
        :param client: the configured connection to Bitbucket.
            If not provided, assumes an Anonymous connection.
        :type client: bitbucket.Client
        :returns: the new repository object.
        :rtype: Repository
        :raises: ValueError
        """
        client = client or Client()
        owner = owner or client.get_username()
        repository_name = repository_name or payload.name
        if not (owner and repository_name):
            raise ValueError('owner and repository_name are required')
        json = payload.validate().build()
        api_url = expand(
            cls.templates['create'], {
                'bitbucket_url': client.get_bitbucket_url(),
                'owner': owner,
                'repository_name': repository_name,
            })
        return cls.post(api_url, json=json, client=client)

    @classmethod
    def fork(
            cls,
            payload,
            repository_name=None,
            owner=None,
            client=None):
        """Forks a repository.

        :param payload: the options for forking repository as a new one
        :type payload: RepositoryForkPayload
        :param repository_name: name of the repository,
            also known as repo_slug. Optional, if provided in the payload.
        :type repository_name: str
        :param owner: the owner of the repository.
            If not provided, assumes the current user.
        :type owner: str
        :param client: the configured connection to Bitbucket.
            If not provided, assumes an Anonymous connection.
        :type client: bitbucket.Client
        :returns: the new repository object.
        :rtype: Repository
        :raises: ValueError
        """
        client = client or Client()
        owner = owner or client.get_username()
        repository_name = repository_name or payload.name
        if not (owner and repository_name):
            raise ValueError('owner and repository_name are required')
        data = payload.validate().build()
        api_url = expand(
            cls.templates['fork'], {
                'bitbucket_url': client.get_bitbucket_url(),
                'owner': owner,
                'repository_name': repository_name,
            })
        return cls.post(api_url, data=data, client=client)

    @staticmethod
    def find_repository_by_name_and_owner(
            repository_name,
            owner=None,
            client=None):
        """
        A convenience method for finding a specific repository.
        In contrast to the pure hypermedia driven method on the Bitbucket
        class, this method returns a Repository object, instead of the
        generator.

        :param repository_name: name of the repository,
            also known as repo_slug.
        :type repository_name: str
        :param owner: the owner of the repository.
            If not provided, assumes the current user.
        :type owner: str
        :param client: the configured connection to Bitbucket.
            If not provided, assumes an Anonymous connection.
        :type client: bitbucket.Client
        :returns: the specific repository object.
        :rtype: Repository
        """
        client = client or Client()
        owner = owner or client.get_username()
        return next(
            Bitbucket(client=client).repositoryByOwnerAndRepositoryName(
                owner=owner,
                repository_name=repository_name))

    @staticmethod
    def find_repository_by_full_name(full_name, client=None):
        """
        A convenience method for finding a specific repository.
        In contrast to the pure hypermedia driven method on the Bitbucket
        class, this method returns a Repository object, instead of the
        generator.

        :param full_name: full name of the repository,
            with both owner and repo_slug separated by slash.
        :type full_name: str
        :param client: the configured connection to Bitbucket.
            If not provided, assumes an Anonymous connection.
        :type client: bitbucket.Client
        :returns: the specific repository object.
        :rtype: Repository
        :raises: TypeError
        """
        client = client or Client()
        if '/' not in full_name:
            raise TypeError(
                "Repository full name must be in the form: username/name")
        owner, repository_name = full_name.split('/')
        return Repository.find_repository_by_name_and_owner(
            owner=owner,
            repository_name=repository_name,
            client=client)

    @staticmethod
    def find_public_repositories(client=None):
        """
        A convenience method for finding public repositories.
        The method is a generator Repository objects.

        :param client: the configured connection to Bitbucket.
            If not provided, assumes an Anonymous connection.
        :type client: bitbucket.Client
        :returns: an iterator over all public repositories.
        :rtype: iterator
        """
        client = client or Client()
        return Bitbucket(client=client).repositoriesThatArePublic()

    @staticmethod
    def find_repositories_by_owner_and_role(
            owner=None,
            role=RepositoryRole.OWNER,
            client=None):
        """
        A convenience method for finding a user's repositories.
        The method is a generator Repository objects.
        When no owner is provided, it uses the currently authenticated user.

        :param owner: the owner of the repository.
            If not provided, assumes the current user.
        :type owner: str
        :param role: the role of the current user on the repositories.
            If not provided, assumes the relationship owner.
        :type role: RepositoryRole
        :param client: the configured connection to Bitbucket.
            If not provided, assumes an Anonymous connection.
        :type client: bitbucket.Client
        :returns: an iterator over all public repositories.
        :rtype: iterator
        """
        client = client or Client()
        owner = owner or client.get_username()
        RepositoryRole(role)
        return Bitbucket(client=client).repositoriesByOwnerAndRole(
            owner=owner,
            role=role)


class RepositoryAdapter(object):
    """A bridge between 1.0 and 2.0 API representations."""

    def __init__(self, data, client=None):
        self.client = client or Client()
        if data.get('full_name') is None:
            # A 1.0 shape has simple owner and name attributes.
            self.owner_name = data.get('owner')
            self.repository_name = data.get('name')
        else:
            # but when constructing from a 2.0 shape
            # then the full_name has to be split.
            self.owner_name, self.repository_name = \
                data['full_name'].split('/')

    def self(self):
        return Repository.find_repository_by_name_and_owner(
            self.repository_name,
            owner=self.owner_name,
            client=self.client)

    def owner(self):
        return User.find_user_by_username(
            self.owner_name,
            client=self.client)


class RepositoryV1(BitbucketBase):
    """Represents a repository in the 1.0 API."""

    id_attribute = 'name'
    links_json = """
{
  "_links": {
    "self": {
      "href": "{+bitbucket_url}/1.0/repositories{/owner,repository_name}"
    },
    "repositories": {
      "href": "{+bitbucket_url}/1.0/repositories{/owner}"
    },
    "changesets": {
      "href": "{+bitbucket_url}/1.0/repositories{/owner,repository_name}/changesets{?limit,start}"
    },
    "deploy_keys": {
      "href": "{+bitbucket_url}/1.0/repositories{/owner,repository_name}/deploy-keys"
    },
    "events": {
      "href": "{+bitbucket_url}/1.0/repositories{/owner,repository_name}/events"
    },
    "followers": {
      "href": "{+bitbucket_url}/1.0/repositories{/owner,repository_name}/followers"
    },
    "issues": {
      "href": "{+bitbucket_url}/1.0/repositories{/owner,repository_name}/issues"
    },
    "integration_links": {
      "href": "{+bitbucket_url}/1.0/repositories{/owner,repository_name}/links"
    },
    "services": {
      "href": "{+bitbucket_url}/1.0/repositories{/owner,repository_name}/services"
    },
    "src": {
      "href": "{+bitbucket_url}/1.0/repositories{/owner,repository_name}/src{/revision,path}"
    },
    "wiki": {
      "href": "{+bitbucket_url}/1.0/repositories{/owner,repository_name}/wiki{/page}"
    }
  }
}
"""  # noqa

    @staticmethod
    def is_type(data):
        return (
            # Categorize as 1.0 structure
            (data.get('resource_uri') is not None) and
            # Categorize as repo-like (repo or snippet)
            (data.get('scm') is not None) and
            # Categorize as repo, not snippet
            (data.get('slug') is not None))

    def __init__(self, data, client=None):
        client = client or Client()
        super(RepositoryV1, self).__init__(data, client)
        self.v2 = RepositoryAdapter(data, client)
        # TODO: Repository src and wiki links are broken.
        # When the uritemplates are filled out,
        # those 2 don't have enough parameters
        # to construct a valid URL.
        expanded_links = self.expand_link_urls(
            bitbucket_url=client.get_bitbucket_url(),
            owner=self.v2.owner_name,
            repository_name=self.v2.repository_name)
        self.links = expanded_links.get('_links', {})
        self.add_remote_relationship_methods(expanded_links)


Client.bitbucket_types.add(Repository)
Client.bitbucket_types.add(RepositoryV1)
